#include "anatcon.h"

#include <qcheckbox.h>
#include <qstring.h>
#include <qlineedit.h>
#include <qpushbutton.h>

#include <iostream>

using namespace std;

anatcon::anatcon( Rinterface *pipe, QWidget* parent, const char* name, 
                  bool modal, WFlags fl )
  : anatconForm( parent, name, modal, fl) {
  
  // set the connection to R
  this->pipe = pipe;

  // set a default value for now - this needs to be fixed!
  cortex = new QString("/home/bic/jason/seconddir/NIH/nih_chp_avg_mid.obj");

  connect(pipe, SIGNAL(newSocketData(QString)), 
          this, SLOT(messageReceived(QString)));
  
  connect(residualsButton, SIGNAL(clicked()),
          this, SLOT(removeRegression()));

  connect(fromPaintButton, SIGNAL(clicked()),
          this, SLOT(startTraceFromLabel()));
  connect(this, SIGNAL(areaTraceReady()),
          this, SLOT(traceFromLabel()));
}


void anatcon::messageReceived( QString message ) {
  if (message.startsWith("ANATCONFILE:")) {
    message = message.remove("ANATCONFILE:");
    message = message.simplifyWhiteSpace();
    cout << "anatcon message: " << message << endl;
    emit newDataFile(message);
  }
  else if (message.startsWith("ANATCONAREA:")) {
    emit areaTraceReady();
  }
}

anatcon::~anatcon() {
  // do nothing
}

void anatcon::traceFromLabel() {
  traceConnectivity(-1);
}

void anatcon::startTraceFromLabel() {
  int paintLabel = paintLabelEdit->text().toInt();
  emit getPaintedVertices( paintLabel );
}

void anatcon::receivePaintedVertices( vector<int> indices ) {
  vector<int>::iterator it = indices.begin();
  QString command;
  QString datatable = dataTableName->text();

  command = "area <- colMeans(";
  command.append(datatable);
  command.append("[c(");
  while (it != indices.end()) {
    command.append(QString::number(*it));
    command.append(", ");
    it++;
  }
  command.append("),])");
  //cout << "CMD: " << command << endl;
  pipe->sendCommand(command);
  pipe->sendRSocketMessage("ANATCONAREA:");
  
}

void anatcon::removeRegression() {
  QString formula = equationEdit->text();
  QString datatable = dataTableName->text();
  QString glim = glimEdit->text();

  QString command;
  command.sprintf("dt.resids <- mni.vertex.residuals(%s, \'%s\', %s)",
                  glim.ascii(), formula.ascii(), datatable.ascii());
  pipe->sendCommand(command, true, "Removing main effects of equation");
  dataTableName->setText("dt.resids");
}

// note vertexNumber should be -1 if it is to be an area trace
void anatcon::traceConnectivity( int vertexNumber ) {

  if (runTracer->isChecked() || vertexNumber == -1) {
    QString corFilename = "/tmp/correlation.vertstats";
    QString command;
    QString savePrevCommand;
    QString dt = dataTableName->text();

    QString tracePoint;
    cout << "Vnum: " << vertexNumber << endl;
    if (vertexNumber == -1) {
      tracePoint = "area";
    }
    else {
      vertexNumber++;
      tracePoint.sprintf("%s[%d,]", dt.ascii(), vertexNumber);
    }

    if (stepTracing->isChecked()) {
      QString dist = minDistanceEdit->text();
      QString val = minValueEdit->text();
      command.sprintf("c <- mni.anatcon.trace(%s, %s\"%s\", min.value = %s, min.distance = %s)", dt.ascii(), tracePoint.ascii(), cortex->ascii(), val.ascii(), dist.ascii());
    }
    else {
      command.sprintf("c <- mni.vertex.correlation(%s, %s)", 
                      dt.ascii(), tracePoint.ascii());
      if (correlationDifference->isChecked()) {
        command.append("\n d <- c - prevCor");
      }
    }
    pipe->sendCommand(command);


    QString writeFileCommand;
    QString variable = "c";
    if (correlationDifference->isChecked())
      variable = "d";
      
    writeFileCommand.sprintf("mni.write.vertex.stats(%s, \"%s\")", 
                             variable.ascii(), corFilename.ascii());
    pipe->sendCommand(writeFileCommand, true, "running correlations");

    savePrevCommand = "prevCor <- c";
    pipe->sendCommand(savePrevCommand);
    
    QString message = "ANATCONFILE:";
    message.append(corFilename);
    pipe->sendRSocketMessage(message);
  }
}

