#include "pointPicker.h"

#include <Inventor/details/SoFaceDetail.h>
#include <Inventor/nodes/SoCoordinate3.h>

#include <iostream>

using namespace std;

pointPicker::pointPicker(){
  // constructor does nothing for now
}

SbBool pointPicker::pickPointFromViewport ( SoNode *root,
                                            const SbViewportRegion &viewport,
                                            const SbVec2s &cursorPosition,
                                            SbBool haveTexture,
                                            const SbVec2f &textureScaleFactor,
                                            float min ) {

  

  SoRayPickAction pickAction(viewport);

  // set an 8 pixed wide region for the search
  pickAction.setPoint(cursorPosition);
  pickAction.setRadius(8.0);
  pickAction.setPickAll(FALSE);
  // start the pick traversal
  pickAction.apply(root);
  const SoPickedPoint *pickedPoint = 
    pickAction.getPickedPoint();


  if (pickedPoint == NULL)
    return FALSE;


  SoFaceDetail *faceDetail = new SoFaceDetail();

  SbVec3f objIntersect = pickedPoint->getObjectPoint();


  // Try to find the closest node to the found item. There really
  // should be a check to make sure that it works the way it is
  // supposed to, having a getByName and all in there.
  SoCoordinate3 *coordNode = new SoCoordinate3;
  coordNode = (SoCoordinate3 *) pickedPoint->getPath()->getTail()->getByName("Vertices");
  faceDetail = (SoFaceDetail *) pickedPoint->getDetail();

  vector <int> indices;
  int closestIndex = -1;
  float minDistance = 1e12;
  cout << "Picker - num points: " << faceDetail->getNumPoints() << endl;
  for (int i = 0; i < faceDetail->getNumPoints(); i++) {
    int pointIndex = faceDetail->getPoint(i)->getCoordinateIndex();
    indices.push_back(pointIndex);
    cout << "Picker - index: " << pointIndex << endl;
    float curDistance = (coordNode->point[pointIndex] - 
                         objIntersect).length();
    if (curDistance < minDistance) {
      closestIndex = pointIndex;
      minDistance = curDistance;
    }
  }
  // cout << "Index: " << closestIndex << endl;

  emit coordinatesPicked(indices);

  const SbVec3f coordinate = pickedPoint->getPoint();
  SbVec4f textureCoord;
  if (haveTexture == TRUE)
    textureCoord = pickedPoint->getObjectTextureCoords();

  /* 
   * cout << "Point: " 
   *      << coordinate[0] << " "
   *      << coordinate[1] << " "
   *      << coordinate[2] << endl;
   */
 
  emit coordinatePicked(coordinate[0],
                        coordinate[1],
                        coordinate[2],
                        closestIndex);
  
  if (haveTexture == TRUE) {

    
         cout << "Texture: "
              << textureCoord[0] << " "
              << textureCoord[1] << " "
              << textureCoord[2] << " "
              << textureCoord[3] << " " 
              << textureScaleFactor[0] << " "
              << textureCoord[0] / textureScaleFactor[0] << endl;
     
    emit texturePicked(textureCoord[1],
                       (textureCoord[1] / textureScaleFactor[1]) + min);
  }

  return FALSE;
}
